#include "I2C1.h"
#include "main.h"

void InitI2C1(void)
{
	TRISBbits.TRISB9 = 0;        //Set RB9 as output (used to solve the issue reported in the Silicon Errata)
	
	
	TX2("  Inicio da configuracao do I2C1");
	
	I2C1CONbits.I2CEN = 0; // Disable I2C Mode
	
	//To compute the Baud Rate Generator reload value, use the following equation:
	//I2C1BRG = (FCY/100000) - (FCY/10000000) - 1;
	I2C1BRG = 0x13; 	//Fosc=4 MHz, SCK=100 kHz
	//I2C1BRG = 0x27; 	//Fosc=8 MHz, SCK=100 kHz
	

	//I2C module configuration
	I2C1CONbits.A10M = 0; //I2CxADD is a 7-bit slave address 
	I2C1CONbits.DISSLW = 1;//Disable slew rate control of the device pin outpup
	
	IFS1bits.MI2C1IF = 0; // Clear all Interrupts
	
	I2C1CONbits.I2CEN = 1; //ENABLING I2C OPERATION
	
	
	LATBbits.LATB9 = 0;		//where LATB9 = the SDA1 pin, (used to solve the issue reported in the Silicon Errata)
	LATBbits.LATB9 = 1;		//(used to solve the issue reported in the Silicon Errata)
	
	unsigned char temp;
	temp = I2C1RCV; // read buffer to clear buffer full
	reset_i2c_bus(); // set bus to idle 
	
	wait(5); //delay cicle before the first read
	
	unsigned char WhoIAm; //dummy register on LIS302DL to check the comunication
	char strADC[100];
	
	WhoIAm = readRegister(0x0F);
	sprintf(strADC,"Who am I (lis302dl should be 59) : %d ", WhoIAm);
	TX2(strADC);
	
	writeRegister(0x20, 0b01000111); //Power on device, enable all axis, and turn off self test - CTRL_REG1
}

void writeRegister(unsigned char regAdd, unsigned char regVal)
{
	//TX2("passei aqu - 11!");
	//while(I2C1STATbits.P == 0); //wait for previous stop bit 
	I2C1CONbits.SEN=1; 			//generate start bus event 
	while(I2C1CONbits.SEN==1); 	//wait for end of start 
	
	//TX2("passei aqu - 12!");
	I2C1TRN= DEVICE_WRITE;				//send device address of slave in 
	while(I2C1STATbits.ACKSTAT==1); 	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 		//wait for end of ACK 

	//TX2("passei aqu - 13!");
	I2C1TRN = regAdd; 					//Write register address
	//I2C1TRN = 0x20;
	while(I2C1STATbits.ACKSTAT==1); 	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 		//wait for end of ACK 

	//TX2("passei aqu - 14!");
	I2C1TRN = regVal; 					//Write the data to register
	//I2C1TRN = 0x47;
	while(I2C1STATbits.ACKSTAT==1); 	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 		//wait for end of ACK 
	
	I2C1CONbits.PEN=1; 				//send stoP 
	while(I2C1CONbits.PEN==1);		//wait for end ofstoP 
	
	//TX2("passei aqu - 1_fim!");
} 

unsigned char readRegister(unsigned char regAdd)
{ 
	while(I2C1STATbits.P == 0); //wait for previous stop bit 
	//TX2("passei aqui - 2 (readReg)");
	I2C1CONbits.SEN=1;				//send start 
	while(I2C1CONbits.SEN==1); 	//wait for end of start 
	
	//TX2("passei aqui - 22");
	I2C1TRN = DEVICE_WRITE; 		//send address to slave read mode 
	while(I2C1STATbits.ACKSTAT==1);	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 	//wait for end of ACK
	
	//TX2("passei aqui - 23");
	I2C1TRN = regAdd; 				//send register address
	//I2C1TRN = 0x29;
	while(I2C1STATbits.ACKSTAT==1);	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 	//wait for end of ACK

	I2C1CONbits.RSEN=1;				//send reapeated Start 
	while(I2C1CONbits.RSEN==1); 	//wait for end of reStart 

	//TX2("passei aqui - 24");
	I2C1TRN = DEVICE_READ; 		//send address to slave read mode 
	while(I2C1STATbits.ACKSTAT==1);	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 	//wait for end of ACK
	
	//TX2("passei aqui - 251");
	I2C1CONbits.RCEN=1; 			//enable receive mode 
	//TX2("passei aqui - 2511");		//********************************************ULTIMO PONTO DE PASSAGEM
	while(I2C1STATbits.RBF==0); 	//wait for received datum <- o PROGRAMA FICA PRESO NESTE WHILE
	//TX2("passei aqui - 252");
	I2C1CONbits.ACKDT=1; 			//send NACK during acknowledge 
	I2C1CONbits.ACKEN=1; 			//send acknowledge (NACK)
	while(I2C1CONbits.ACKEN==1); 	//wait for end of acknowledge (NACK) 
	
	//TX2("passei aqui - 26");
	I2C1CONbits.PEN=1; 				//send stoP 
	while(I2C1CONbits.PEN==1);		//wait for end ofstoP 
	

	
	return(I2C1RCV); 
	
	//TX2("passei aqui - 2_fim");
}

//Resets the I2C bus to Idle
void reset_i2c_bus(void)
{
	//initiate stop bit
	I2C1CONbits.PEN = 1;

	//wait for hardware clear of stop bit
	while (I2C1CONbits.PEN);

	//clear all interrupt flags and warning flags
	I2C1CONbits.RCEN = 0;
	IFS1bits.MI2C1IF = 0; // Clear Interrupt
	I2C1STATbits.IWCOL = 0;
	I2C1STATbits.BCL = 0;
}

